import 'package:flutter/material.dart';
import 'package:flutter_waya/flutter_waya.dart';
import 'package:flutter_waya/src/common/CommonWidget.dart';
import 'package:flutter_waya/src/constant/WayColor.dart';

showToast(message, {Duration closeDuration}) {
  AlertTools.showToast(message.toString(), closeDuration: closeDuration);
}

class AlertTools {
  static Duration _duration = Duration(milliseconds: 1500);


  ///设置全局弹窗时间
  static void setToastDuration(Duration duration) {
    _duration = duration;
  }

  static void showToast(String message,
      {Color backgroundColor,
        BoxDecoration boxDecoration,
        GestureTapCallback onTap,
        TextStyle textStyle,
        Duration closeDuration}) {
    OverlayTools.show(
        AlertBase(
            backgroundColor: backgroundColor ?? getColors(transparent),
            onTap: onTap ?? () {},
            child: Container(
              margin: EdgeInsets.symmetric(horizontal: Tools.getWidth() / 5, vertical: Tools.getHeight() / 4),
              decoration:
              boxDecoration ?? BoxDecoration(color: getColors(black90), borderRadius: BorderRadius.circular(5)),
              padding: EdgeInsets.all(Tools.getWidth(10)),
              child: CommonWidget.textDefault(message, color: getColors(white)),
            )),
        closeDuration: closeDuration ?? _duration);
  }

  static void close() {
    OverlayTools.close();
  }

  ///关闭所有
  static void closeAll() {
    OverlayTools.close();
  }

  static void alertSureCancel(Widget widget, {
    GestureTapCallback sureTap,
    GestureTapCallback cancelTap,
    String cancelText,
    String sureText,
    Widget sure,
    Widget cancel,
    Color backgroundColor,
    TextStyle cancelTextStyle,
    TextStyle sureTextStyle,
    double height,
    bool isDismissible: true,
    EdgeInsetsGeometry padding,
    AlignmentGeometry alignment,
    EdgeInsetsGeometry margin,
    Decoration decoration,
    bool animatedOpacity,
    bool gaussian,
    bool addMaterial, //是否添加Material Widget 部分组件需要基于Material
  }) {
    var alert = AlertSureCancel(
      backsideTap: () {
        if (isDismissible) close();
      },
      addMaterial: addMaterial,
      animatedOpacity: animatedOpacity,
      gaussian: gaussian,
      content: widget,
      sureTap: sureTap ?? close,
      cancelTap: cancelTap ?? close,
      decoration: decoration,
      margin: margin,
      alignment: alignment,
      cancelText: cancelText,
      sureText: sureText,
      height: height,
      cancelTextStyle: cancelTextStyle,
      sureTextStyle: sureTextStyle,
      sure: sure,
      cancel: cancel,
      backgroundColor: backgroundColor,
      padding: padding,
    );
    OverlayTools.show(alert);
  }

  static void showLoading({
    String text,
    double value,
    bool gaussian,
    Color backgroundColor,
    Animation<Color> valueColor,
    double strokeWidth,
    String semanticsLabel,
    String semanticsValue,
    LoadingType loadingType,
    TextStyle textStyle,
  }) {
    var loading = Loading(
      gaussian: gaussian,
      text: text,
      value: value,
      backgroundColor: backgroundColor,
      valueColor: valueColor,
      strokeWidth: strokeWidth ?? 4.0,
      semanticsLabel: semanticsLabel,
      semanticsValue: semanticsValue,
      loadingType: loadingType ?? LoadingType.circular,
      textStyle: textStyle,
    );
    OverlayTools.show(loading);
  }
}
